@@ -13,6 +13,7 @@
 #include "keycache.h"
 #include "keycache_p.h"
 
+#include "kleo/enum.h"
 #include "kleo/keygroup.h"
 #include "kleo/predicates.h"
 #include "kleo/stl_util.h"
@@ -236,7 +237,8 @@ public:
 
         // collect the key fingerprints for all groups read from the configuration
         QMap<QString, QStringList> fingerprints;
-        for (const QString &value: entry->stringValueList()) {
+        const auto stringValueList = entry->stringValueList();
+        for (const QString &value: stringValueList) {
             const QStringList split = value.split(QLatin1Char('='));
             if (split.size() != 2) {
                 qCDebug (LIBKLEO_LOG) << "Ignoring invalid group config:" << value;
@@ -515,7 +517,7 @@ void KeyCache::setGroupsConfig(const QString &filename)
 
 void KeyCache::enableFileSystemWatcher(bool enable)
 {
-    for (const auto &i : qAsConst(d->m_fsWatchers)) {
+    for (const auto &i : std::as_const(d->m_fsWatchers)) {
         i->setEnabled(enable);
     }
 }
@@ -1170,7 +1172,7 @@ void KeyCache::saveConfigurableGroups(const std::vector<KeyGroup> &groups)
                             newGroups.begin(), newGroups.end(),
                             std::back_inserter(removedGroups),
                             &compareById);
-        for (const auto &group : qAsConst(removedGroups)) {
+        for (const auto &group : std::as_const(removedGroups)) {
             qCDebug(LIBKLEO_LOG) << "Removing group" << group;
             d->remove(group);
         }
@@ -1181,7 +1183,7 @@ void KeyCache::saveConfigurableGroups(const std::vector<KeyGroup> &groups)
                               oldGroups.begin(), oldGroups.end(),
                               std::back_inserter(updatedGroups),
                               &compareById);
-        for (const auto &group : qAsConst(updatedGroups)) {
+        for (const auto &group : std::as_const(updatedGroups)) {
             qCDebug(LIBKLEO_LOG) << "Updating group" << group;
             d->update(group);
         }
@@ -1192,7 +1194,7 @@ void KeyCache::saveConfigurableGroups(const std::vector<KeyGroup> &groups)
                             oldGroups.begin(), oldGroups.end(),
                             std::back_inserter(addedGroups),
                             &compareById);
-        for (const auto &group : qAsConst(addedGroups)) {
+        for (const auto &group : std::as_const(addedGroups)) {
             qCDebug(LIBKLEO_LOG) << "Adding group" << group;
             d->insert(group);
         }
@@ -1393,7 +1395,7 @@ void KeyCache::insert(const std::vector<Key> &keys)
     by_keygrip.swap(d->by.keygrip);
     by_chainid.swap(d->by.chainid);
 
-    for (const Key &key : qAsConst(sorted)) {
+    for (const Key &key : std::as_const(sorted)) {
         d->m_pgpOnly &= key.protocol() == GpgME::OpenPGP;
         Q_EMIT added(key);
     }
@@ -1634,17 +1636,17 @@ static bool subkeyIsOk(const Subkey &s)
 
 namespace
 {
-time_t creationTimeOfNewestSuitableSubKey(const Key &key, bool needSign, bool needEncrypt)
+time_t creationTimeOfNewestSuitableSubKey(const Key &key, KeyUsage usage)
 {
     time_t creationTime = 0;
     for (const Subkey &s: key.subkeys()) {
         if (!subkeyIsOk(s)) {
             continue;
         }
-        if (needSign && !s.canSign()) {
+        if (usage == KeyUsage::Sign && !s.canSign()) {
             continue;
         }
-        if (needEncrypt && !s.canEncrypt()) {
+        if (usage == KeyUsage::Encrypt && !s.canEncrypt()) {
             continue;
         }
         if (s.creationTime() > creationTime) {
@@ -1662,40 +1664,32 @@ struct BestMatch
 };
 }
 
-std::vector<GpgME::Key> KeyCache::findBestByMailBox(const char *addr, GpgME::Protocol proto,
-                                                    bool needSign, bool needEncrypt) const
+GpgME::Key KeyCache::findBestByMailBox(const char *addr, GpgME::Protocol proto, KeyUsage usage) const
 {
     d->ensureCachePopulated();
-    std::vector<GpgME::Key> ret;
     if (!addr) {
-        return ret;
-    }
-    if (proto == Protocol::OpenPGP) {
-        // Groups take precedence over automatic keys. We return all here
-        // so if a group key for example is expired we can show that.
-        ret = getGroupKeys(QString::fromUtf8(addr));
-        if (!ret.empty()) {
-            return ret;
-        }
+        return {};
     }
+
     // support lookup of email addresses enclosed in angle brackets
     QByteArray address(addr);
     if (address[0] == '<' && address[address.size() - 1] == '>') {
         address = address.mid(1, address.size() - 2);
     }
     address = address.toLower();
+
     BestMatch best;
     for (const Key &k: findByEMailAddress(address.constData())) {
         if (proto != Protocol::UnknownProtocol && k.protocol() != proto) {
             continue;
         }
-        if (needEncrypt && !k.canEncrypt()) {
+        if (usage == KeyUsage::Encrypt && !k.canEncrypt()) {
             continue;
         }
-        if (needSign && (!k.canSign() || !k.hasSecret())) {
+        if (usage == KeyUsage::Sign && (!k.canSign() || !k.hasSecret())) {
             continue;
         }
-        const time_t creationTime = creationTimeOfNewestSuitableSubKey(k, needSign, needEncrypt);
+        const time_t creationTime = creationTimeOfNewestSuitableSubKey(k, usage);
         if (creationTime == 0) {
             // key does not have a suitable (and usable) subkey
             continue;
@@ -1723,14 +1717,60 @@ std::vector<GpgME::Key> KeyCache::findBestByMailBox(const char *addr, GpgME::Pro
             }
         }
     }
-    ret.push_back(best.key);
-    return ret;
+
+    return best.key;
+}
+
+namespace
+{
+template<typename T>
+bool allKeysAllowUsage(const T &keys, KeyUsage usage)
+{
+    switch(usage) {
+    case KeyUsage::AnyUsage:
+        return true;
+    case KeyUsage::Sign:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canSign));
+    case KeyUsage::Encrypt:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canEncrypt));
+    case KeyUsage::Certify:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canCertify));
+    case KeyUsage::Authenticate:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canAuthenticate));
+    }
+    qCDebug(LIBKLEO_LOG) << __func__ << "called with invalid usage" << int(usage);
+    return false;
+}
+
+template<typename T>
+bool allKeysHaveProtocol(const T &keys, Protocol protocol)
+{
+    return std::all_of(std::begin(keys), std::end(keys), [protocol] (const auto &key) { return key.protocol() == protocol; });
+}
+}
+
+KeyGroup KeyCache::findGroup(const QString &name, Protocol protocol, KeyUsage usage) const
+{
+    d->ensureCachePopulated();
+
+    Q_ASSERT(usage == KeyUsage::Sign || usage == KeyUsage::Encrypt);
+    for (const auto &group : std::as_const(d->m_groups)) {
+        if (group.name() == name) {
+            const KeyGroup::Keys &keys = group.keys();
+            if (allKeysAllowUsage(keys, usage)
+                && (protocol == UnknownProtocol || allKeysHaveProtocol(keys, protocol))) {
+                return group;
+            }
+        }
+    }
+
+    return {};
 }
 
 std::vector<Key> KeyCache::getGroupKeys(const QString &groupName) const
 {
     std::vector<Key> result;
-    for (const KeyGroup &g : qAsConst(d->m_groups)) {
+    for (const KeyGroup &g : std::as_const(d->m_groups)) {
         if (g.name() == groupName) {
             const KeyGroup::Keys &keys = g.keys();
             std::copy(keys.cbegin(), keys.cend(), std::back_inserter(result));
@@ -1741,6 +1781,23 @@ std::vector<Key> KeyCache::getGroupKeys(const QString &groupName) const
     return result;
 }
 
+void KeyCache::setKeys(const std::vector<GpgME::Key>& keys)
+{
+    // disable regular key listing and cancel running key listing
+    setRefreshInterval(0);
+    cancelKeyListing();
+    clear();
+    insert(keys);
+    d->m_initalized = true;
+    Q_EMIT keyListingDone(KeyListResult());
+}
+
+void KeyCache::setGroups(const std::vector<KeyGroup>& groups)
+{
+    Q_ASSERT(d->m_initalized && "Call setKeys() before setting groups");
+    d->m_groups = groups;
+    Q_EMIT keysMayHaveChanged();
+}
+
 #include "moc_keycache_p.cpp"
 #include "moc_keycache.cpp"
-
