@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEO_KEYLISTVIEW_H__
-#define __KLEO_KEYLISTVIEW_H__
+#pragma once
 
 #include "kleo_export.h"
 
@@ -201,4 +200,3 @@ private:
 };
 }
 
-#endif // __KLEO_KEYLISTVIEW_H__
