@@ -30,6 +30,40 @@ Q_DECLARE_METATYPE(GpgME::Key)
 
 namespace
 {
+class SortFilterProxyModel : public KeyListSortFilterProxyModel
+{
+    Q_OBJECT
+
+public:
+    using KeyListSortFilterProxyModel::KeyListSortFilterProxyModel;
+
+    void setAlwaysAcceptedKey(const QString &fingerprint)
+    {
+        if (fingerprint == mFingerprint) {
+            return;
+        }
+        mFingerprint = fingerprint;
+        invalidate();
+    }
+
+protected:
+    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override
+    {
+        if (!mFingerprint.isEmpty()) {
+            const QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
+            const auto fingerprint = sourceModel()->data(index, KeyList::FingerprintRole).toString();
+            if (fingerprint == mFingerprint) {
+                return true;
+            }
+        }
+
+        return KeyListSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
+    }
+
+private:
+    QString mFingerprint;
+};
+
 class ProxyModel : public QSortFilterProxyModel
 {
     Q_OBJECT
@@ -313,7 +347,7 @@ public:
 
     /* Updates the current key with the default key if the key matches
      * the current key filter. */
-    void updateWithDefaultKey() const {
+    void updateWithDefaultKey() {
         GpgME::Protocol filterProto = GpgME::UnknownProtocol;
 
         const auto filter = dynamic_cast<const DefaultKeyFilter*> (sortFilterProxy->keyFilter().get());
@@ -328,18 +362,52 @@ public:
             // Fallback to unknown protocol
             defaultKey = defaultKeys.value (GpgME::UnknownProtocol);
         }
+        // make sure that the default key is not filtered out unless it has the wrong protocol
+        if (filterProto == GpgME::UnknownProtocol) {
+            sortFilterProxy->setAlwaysAcceptedKey(defaultKey);
+        } else {
+            const auto key = KeyCache::instance()->findByFingerprint(defaultKey.toLatin1().constData());
+            if (!key.isNull() && key.protocol() == filterProto) {
+                sortFilterProxy->setAlwaysAcceptedKey(defaultKey);
+            } else {
+                sortFilterProxy->setAlwaysAcceptedKey({});
+            }
+        }
         q->setCurrentKey(defaultKey);
     }
 
+    void storeCurrentSelectionBeforeModelChange()
+    {
+        keyBeforeModelChange = q->currentKey();
+        customItemBeforeModelChange = q->currentData();
+    }
+
+    void restoreCurrentSelectionAfterModelChange()
+    {
+        if (!keyBeforeModelChange.isNull()) {
+            q->setCurrentKey(keyBeforeModelChange);
+        } else if (customItemBeforeModelChange.isValid()) {
+            const auto index = q->findData(customItemBeforeModelChange);
+            if (index != -1) {
+                q->setCurrentIndex(index);
+            } else {
+                updateWithDefaultKey();
+            }
+        }
+    }
+
     Kleo::AbstractKeyListModel *model = nullptr;
-    Kleo::KeyListSortFilterProxyModel *sortFilterProxy = nullptr;
+    SortFilterProxyModel *sortFilterProxy = nullptr;
     ProxyModel *proxyModel = nullptr;
     std::shared_ptr<Kleo::KeyCache> cache;
     QMap<GpgME::Protocol, QString> defaultKeys;
     bool wasEnabled = false;
     bool useWasEnabled = false;
-    bool secretOnly;
+    bool secretOnly = false;
+    bool initialKeyListingDone = false;
     QString mPerfectMatchMbox;
+    GpgME::Key keyBeforeModelChange;
+    QVariant customItemBeforeModelChange;
 
 private:
     KeySelectionCombo * const q;
@@ -360,18 +428,18 @@ KeySelectionCombo::KeySelectionCombo(bool secretOnly, QWidget* parent)
     d->model = Kleo::AbstractKeyListModel::createFlatKeyListModel(this);
     d->secretOnly = secretOnly;
 
-    d->sortFilterProxy = new Kleo::KeyListSortFilterProxyModel(this);
+    d->sortFilterProxy = new SortFilterProxyModel(this);
     d->sortFilterProxy->setSourceModel(d->model);
 
     d->proxyModel = new ProxyModel(this);
     d->proxyModel->setSourceModel(d->sortFilterProxy);
 
     setModel(d->proxyModel);
-    connect(this, static_cast<void(KeySelectionCombo::*)(int)>(&KeySelectionCombo::currentIndexChanged),
+    connect(this, QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, [this](int row) {
                 if (row >= 0 && row < d->proxyModel->rowCount()) {
                     if (d->proxyModel->isCustomItem(row)) {
-                        Q_EMIT customItemSelected(d->proxyModel->index(row, 0).data(Qt::UserRole));
+                        Q_EMIT customItemSelected(currentData(Qt::UserRole));
                     } else {
                         Q_EMIT currentKeyChanged(currentKey());
                     }
@@ -380,6 +448,19 @@ KeySelectionCombo::KeySelectionCombo(bool secretOnly, QWidget* parent)
 
     d->cache = Kleo::KeyCache::mutableInstance();
 
+    connect(model(), &QAbstractItemModel::rowsAboutToBeInserted,
+            this, [this] () { d->storeCurrentSelectionBeforeModelChange(); });
+    connect(model(), &QAbstractItemModel::rowsInserted,
+            this, [this] () { d->restoreCurrentSelectionAfterModelChange(); });
+    connect(model(), &QAbstractItemModel::rowsAboutToBeRemoved,
+            this, [this] () { d->storeCurrentSelectionBeforeModelChange(); });
+    connect(model(), &QAbstractItemModel::rowsRemoved,
+            this, [this] () { d->restoreCurrentSelectionAfterModelChange(); });
+    connect(model(), &QAbstractItemModel::modelAboutToBeReset,
+            this, [this] () { d->storeCurrentSelectionBeforeModelChange(); });
+    connect(model(), &QAbstractItemModel::modelReset,
+            this, [this] () { d->restoreCurrentSelectionAfterModelChange(); });
+
     QTimer::singleShot(0, this, &KeySelectionCombo::init);
 }
 
@@ -394,8 +475,10 @@ void KeySelectionCombo::init()
             this, [this]() {
                     // Set useKeyCache ensures that the cache is populated
                     // so this can be a blocking call if the cache is not initialized 
-                    d->model->useKeyCache(true, d->secretOnly ? KeyList::SecretKeysOnly : KeyList::AllKeys);
-                    d->proxyModel->removeCustomItem(QStringLiteral("-libkleo-loading-keys"));
+                    if (!d->initialKeyListingDone) {
+                        d->model->useKeyCache(true, d->secretOnly ? KeyList::SecretKeysOnly : KeyList::AllKeys);
+                        d->proxyModel->removeCustomItem(QStringLiteral("-libkleo-loading-keys"));
+                    }
 
                     // We use the useWasEnabled state variable to decide if we should
                     // change the enable / disable state based on the keylist done signal.
@@ -413,7 +496,10 @@ void KeySelectionCombo::init()
             });
 
     connect(this, &KeySelectionCombo::keyListingFinished, this, [this]() {
-            d->updateWithDefaultKey();
+            if (!d->initialKeyListingDone) {
+                d->updateWithDefaultKey();
+                d->initialKeyListingDone = true;
+            }
         });
 
     if (!d->cache->initialized()) {
@@ -460,11 +546,11 @@ GpgME::Key Kleo::KeySelectionCombo::currentKey() const
 
 void Kleo::KeySelectionCombo::setCurrentKey(const GpgME::Key &key)
 {
-    const int idx = findData(QVariant::fromValue(key), KeyList::KeyRole, Qt::MatchExactly);
+    const int idx = findData(QString::fromLatin1(key.primaryFingerprint()), KeyList::FingerprintRole, Qt::MatchExactly);
     if (idx > -1) {
         setCurrentIndex(idx);
-    } else {
-        d->selectPerfectIdMatch();
+    } else if (!d->selectPerfectIdMatch()) {
+        d->updateWithDefaultKey();
     }
     setToolTip(currentData(Qt::ToolTipRole).toString());
 }
@@ -474,7 +560,9 @@ void Kleo::KeySelectionCombo::setCurrentKey(const QString &fingerprint)
     const auto cur = currentKey();
     if (!cur.isNull() && !fingerprint.isEmpty() &&
         fingerprint == QLatin1String(cur.primaryFingerprint())) {
-        // Already set
+        // already set; still emit a changed signal because the current key may
+        // have become the item at the current index by changes in the underlying model
+        Q_EMIT currentKeyChanged(cur);
         return;
     }
     const int idx = findData(fingerprint, KeyList::FingerprintRole, Qt::MatchExactly);
@@ -518,6 +606,11 @@ void KeySelectionCombo::prependCustomItem(const QIcon &icon, const QString &text
     prependCustomItem(icon, text, data, QString());
 }
 
+void KeySelectionCombo::removeCustomItem(const QVariant &data)
+{
+    d->proxyModel->removeCustomItem(data);
+}
+
 void Kleo::KeySelectionCombo::setDefaultKey(const QString &fingerprint, GpgME::Protocol proto)
 {
     d->defaultKeys.insert(proto, fingerprint);
